#!python3
#psychowoife

import re
import os
import sys
import getopt
import hashlib
import shutil
import pysftp
import subprocess

##globals
version = '2020.05.20'
git_ver_pattern = r'-(?P<version>\d{5})-'
#start game
game_start_pat = '<game'
game_stop_pat = '<\/game>'
game_name_pat = 'game\s+name=\s*"(?P<name>[^"]*)"'
desc_pat = '<description>(?P<desc>[^"]*)<\/description>'
rom_name_pat = '<rom\s+name\s*=\s*"(?P<rom_name>[^"]*)"\s*size\s*=\s*"(?P<size>\d*)\s*"\s*crc\s*=\s*"(?P<crc>[\dabcdefABCDEF]*)\s*"\s*md5\s*=\s*"\s*(?P<md5>[\dabcdefABCDEF]*)\s*"\s*sha1\s*=\s*"\s*(?P<sha1>[\dabcdefABCDEF]*)\s*"\s*(status\s*=\s*"(?P<status>[^"]*)"|)\s*serial\s*=\s*"\s*(?P<serial>[^"]*)\s*"'


class DatafileFormatError(Exception):
	pass


##prints message how to use this script
def usage():
   print('check if new build is necessary')
   print('usage: ' + os.path.basename(sys.argv[0]) + ' -a <host> -u <user> -p <passwd> -f <host_file> -g <git_link> -l <sftp_path>[-d] [-v]\n')
   print('arguments:')
   print('\t-a --host=\t\thost to connect')
   print('\t-u --user=\t\tusername')
   print('\t-p --passwd=\t\tpassword')
   print('\t-f --host_file=\t\tknown host file')
   print('\t-g --git_link=\t\tpath to git repo')
   print('\t-l --sftp_path=\t\tpath to builds')
   print('optional:')
   print('\t-d --debug\t\t')
   print('\t-v --version\t\tprints version of script')


##prints error message and exits with error code
def err(str, type):
   error = 'ERROR: '
   if type == 0:
      print(error + str + ': argument missing')
   elif type == 1:
      print(error + str + ': file not found')
   elif type == 2:
      print(error + 'wrong parameter')
   elif type == 3:
      print(error + 'to few arguments')
   elif type == 4:
      print(error + 'unknown arguments')
   elif type == 5:
      print(error + ': path not found')
   usage()
   sys.exit(1)
   

##print version
def print_version():
   print(version)


##parses command line arguments and returns dictionary contaning all arguments
def parse_args():
   #try parsing arguments according to defined format
   try:
      opts, args = getopt.getopt(sys.argv[1:], 'a:u:p:f:g:l:c:dvh', ['host=', 'user=', 'passwd=', 'host_file=', 'git_path=', 'sftp_path=', 'cp_path=', 'debug', 'version', 'help'])
   except getopt.GetoptError:
      err('',2)   #parsing failed
      
   debug = False
   sftp_host = ''
   sftp_user = ''
   sftp_passwd = ''
   sftp_path = ''
   known_hosts = ''
   git_path = ''
   cp_path = ''
   
   #check if arguments even present
   if not opts:
      err('', 3)
   #get each value for given argument
   for opt, arg in opts:
      if opt in ('-h', '--help'):
         usage()
         sys.exit(1)
      if opt in ('-v', '--version'):
         print_version()
         sys.exit(0)
      elif opt in ('-a', '--host'):
         sftp_host = arg
      elif opt in ('-u', '--user'):
         sftp_user = arg
      elif opt in ('-p', '--passwd'):
         sftp_passwd = arg
      elif opt in ('-f', '--host_file'):
         if not os.path.isfile(arg):
            err(arg, 1)
         known_hosts = arg
      elif opt in ('-g', '--git_path'):
         if not os.path.isdir(arg):
            err(arg, 5)
         git_path = arg
      elif opt in ('-c', '--cp_path'):
         cp_path = arg
      elif opt in ('-l', '--sftp_path'):
         sftp_path = arg
      elif opt in ('-d', '--debug'):
         debug = True
      else:
         err(arg, 4) #exit script if false parameter is detected

   if not sftp_host:
      err('sftp_host', 0)
   if not sftp_user:
      err('sftp_user', 0)
   if not sftp_passwd:
      err('sftp_passwd', 0)
   if not known_hosts:
      err('known_hosts', 0)
   if not sftp_host:
      err('sftp_host', 0)
   if not git_path:
      err('git_path', 0)
   if not sftp_path:
      err('sftp_path', 0)
   
   param = dict()
   param['sftp_host'] = sftp_host
   param['sftp_user'] = sftp_user
   param['sftp_passwd'] = sftp_passwd
   param['known_hosts'] = known_hosts
   param['git_path'] = git_path
   param['sftp_path'] = sftp_path
   param['path_to_cp'] = cp_path
   param['debug'] = debug
   
   return param
   
   
##calculate md5 chesksum of file
def calc_md5(filename):
    hash_md5 = hashlib.md5()
    with open(filename, "rb") as file:
        for chunk in iter(lambda: file.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()
    
    
progressDict={}
progressEveryPercent=10

##
##for i in range(0,101):
##    if i%progressEveryPercent==0:
##       progressDict[str(i)]=""
##
##def printProgressDecimal(x,y):
##    if int(100*(int(x)/int(y))) % progressEveryPercent ==0 and progressDict[str(int(100*(int(x)/int(y))))]=="":
##        print("{}% ({} Transfered(B)/ {} Total File Size(B))".format(str("%.2f" %(100*(int(x)/int(y)))),x,y))
##        progressDict[str(int(100*(int(x)/int(y))))]="1"
##   
    
def sftp_connect(myhost, myname, mypasswd, known_hosts='known_hosts'):
	cnopts = pysftp.CnOpts()
	cnopts.hostkeys.load(known_hosts)
	return pysftp.Connection(host=myhost, username=myname, password=mypasswd, cnopts=cnopts)


def get_last_build(sftp, bin_path='/jenkins/owrt'):
	sftp.cwd(bin_path)
	directory_structure = sftp.listdir_attr()

	files = []
	for attr in directory_structure:
		files.append(attr.filename)

	files.sort()
	return files[-1]
	
	
def cp_build_to_server(sftp, sftp_path, dir_to_cp, foldername):
	#print(sftp_path)
	#print(dir_to_cp)
	#print(os.getcwd())
	#print(foldername)
	#print(sftp_path + foldername)
	sftp.makedirs(sftp_path + foldername, mode=755)
	#list = sftp.listdir()
	#print(list)
	#sftp.chdir(foldername)
	#print(sftp.pwd)
	sftp.put_r(dir_to_cp, sftp_path + foldername) ##, callback=lambda x,y: printProgressDecimal(x,y))
	return


def build_needed(current, last_build_version):
	if current > last_build_version:
		return True
	else:
		return False
	#list = []
	#list.append(current)
	#list.append(last_build_version)
	#list.sort()
	#if list
	
	
def main():
	params = parse_args()
	sftp = sftp_connect(params['sftp_host'], params['sftp_user'], params['sftp_passwd'], known_hosts=params['known_hosts'])
	last = get_last_build(sftp, bin_path=params['sftp_path'])
	#print(last)
	os.chdir(params['git_path'])
	out = subprocess.check_output(['git', 'describe']).decode(sys.stdout.encoding).strip()
	match = re.search(git_ver_pattern, out)
	current = '00000'
	if match:
		current = match.group('version')
	#print(last)
	#print(current)
	build= build_needed('r'+current, last)
	print(build)
	if params['path_to_cp'] != '' and build:
		#print(params['path_to_cp'])
		cp_build_to_server(sftp, params['sftp_path'], params['path_to_cp'], 'r'+current)
	sftp.close()

if __name__ == "__main__":
   main()
